

/*
  DIY polygraph with KMR-1.8 TFT display connected:
  
  UNO +5V to display pin 2 (VCC)
  UNO 0V (GND) to display pin 1 (GND)
  UNO +5V through a 22 Ohm resistor to display pin 15 (LED+)
  UNO 0V (GND) to display pin 16 (LED-)
  UNO digital pin 7 through a 1K2 resistor to display pin 6 (RESET), add a 2K2 resistor from display pin 6 to GND
  UNO digital pin 8 through a 1K2 resistor to display pin 7 (A0), add a 2K2 resistor from display pin 7 to GND
  UNO digital pin 11 through a 1K2 resistor to display pin 8 (SDA), add a 2K2 resistor from display pin 8 to GND
  UNO digital pin 13 through a 1K2 resistor to display pin 9 (SCL), add a 2K2 resistor from display pin 9 to GND
  UNO digital pin 9 through a 1K2 resistor to display pin 10 (CS), add a 2K2 resistor from display pin 10 to GND

  xinda heartbeat sensor on A0
  skin resistance measured on pin A1
  piezo speaker on pin 3
  screen mode setting on pin A5

  see https://www.niklasroy.com/diypolygraph for further details

*/

#include <TFT_ST7735.h>                                 // Adafruit tft screen library
TFT_ST7735 tft = TFT_ST7735();                          // create tft object


//........................................................ io pins
const int pulseInputPin = A0;                           // input pin for pulse sensor
const int GSRinputPin   = A1;                           // input pin for galvanic skin response
const int screenModePin = A5;                           // input pin for display settings: show only gsr curve if connected to gnd
const int speakerPin    = 3;                            // output pin for piezo speaker

//........................................................ general globals
boolean   displayPulse, pDisplayPulse;                  // screen settings
long      nextCycle = 0;                                // used for consistent 10Hz loop timing
int       loopCount = 0;                                // general loop counter
int       freqOscillator = 20;                          // simple sound oscillator

//........................................................ gsr
float     gsrMOhms[160];                                // galvanic skin response - ringbuffer - saves last 160 values of skin resistance in MOhms
double    gsrMin = 0;                                   // minimum value in MOhms of last 20 samples (filtered)
double    gsrMax = 0;                                   // maximum value in MOhms of last 20 samples (filtered)
double    gsrAverage = 0;                               // latest average of gsr which will be displayed as absolute value
double    gsrOldAverage = 0;                            // previous gsr average
float     gsrRangeDisplay = 0;                          // gsr range as it is displayed
float     gsrRange;
long      gsrUpdate = 0;                                // used to limit how often absolute value display can change
boolean   gsrValueUpdate;                               // becomes true if a new gsr value can be displayed
int       gsrArrow = 0;                                 // keeps track of arrows which are drawn on the screen - used for beeping (+1=up, -1=down)
int       pGsrArrow = 0;

//........................................................ pulse
int       pulseSamples[80];                             // ringbuffer which saves raw samples from analog pin
float     pulseMin = 0;                                 // minimum value in MOhms of last 20 samples (filtered)
float     pulseMax = 0;                                 // maximum value in MOhms of last 20 samples (filtered)
int       pulseThreshold;                               // threshold calculated from min and max
boolean   heartBeat = false;                            // heart beat detection
boolean   pHeartBeat = false;                           // heart beat value in previous cycle
long      beatTime[20];                                 // times in millis when last 20 peaks were detected
float     bpm, oldBpm;                                  // heart rate based on duration of last 10 pulses - is updated with each pulse
int       heartArrow = 0;                               // keeps track of arrows which are drawn on the screen - used for beeping (+1=up, -1=down)
int       pHeartArrow = 0;

////////////////////////////////////////////////////////// SETUP:
void setup() {

  pinMode(speakerPin, OUTPUT);                          // set speaker pin as output
  pinMode(screenModePin, INPUT_PULLUP);                 // set display mode pin as input pin
  pinMode(13, OUTPUT);                                  // LED

  tft.init();                                           // initialize a tft screen with ST7735S chip
  tft.setRotation(1);                                   // rotate screen: x=horizontal, y=vertical axis
  tft.fillRect(-2, -2, 161, 128, TFT_BLACK);            // blank screen
  tft.setTextSize(2);                                   // large text size
  // ----------------------------------------------------- startup animation

  tft.setCursor(25, 55);
  tft.setTextColor(color(0,  .9, .9));
  tft.print("POLY");
  tft.setCursor(77, 55);
  tft.setTextColor(color(0, 1, 1));
  tft.print("GRAPH");


  // ----------------------------------------------------- startup beeps & led blinks

  tone(speakerPin, 1500);

  delay(150);
  tone(speakerPin, 500);
  delay(100);
  tone(speakerPin, 2000);
  delay(200);
  noTone(speakerPin);
  delay(700);

  tft.fillRect(-2, -2, 161, 128, TFT_BLACK);            // blank screen
  tft.setTextSize(1);                                   // small text sizeon

  Serial.begin(9600);

}

////////////////////////////////////////////////////////// LOOP:
void loop() {
  loopCount++;                                          // general loop counter

  // ----------------------------------------------------- screen settings
  pDisplayPulse = displayPulse;                         // remember previous scrren setting
  displayPulse = digitalRead(screenModePin);            // read screen setting
  if (displayPulse != pDisplayPulse) {                  // if screen mode has changed:
    tft.fillRect(-2, -2, 161, 128, TFT_BLACK);          // clear screen
  }

  // ----------------------------------------------------- consistent timing: limit loop execution speed to 10 Hertz
  int freqIn = analogRead(GSRinputPin);
  while (millis() < nextCycle) {                        // wait until 100 millis have passed since last loop
    if (gsrMOhms[0] < 30) {                             // if fingers are on sensor, play some sounds
      
      int freqOut = pow(freqIn, 2) / 1500 + 60 + freqOscillator;
      tone(speakerPin, freqOut);
      freqOscillator *= -1;
      delay(1);
    } else {                                            // if no fingers are on gsr sensor: stop all sounds
      noTone(speakerPin);
    }
  }
  Serial.print("0,1023,");
  Serial.println(freqIn);
  nextCycle = millis() + 100L;

  // ----------------------------------------------------- read galvanic skin response
  readGsr();
  // ----------------------------------------------------- read pulse sensor
  readPulseSensor();
  detectHeartBeat();                                    // peak detection

  // ----------------------------------------------------- draw pulse curve (depending on screen mode)
  if (displayPulse) drawPulse();

  // ----------------------------------------------------- draw galvanic skin response curve
  drawGsr();
}
